unit BMP_To_Jpeg_Thread_Unit;

interface

uses
  Windows, SysUtils, Classes, Graphics, IJL;

type
  EIJLException = class(Exception);

type
  TIJLScale=(ijlFull, ijlHalf, ijlQuarter, ijlEighth);


type
  TBMP_To_Jpeg_Thread = class(TThread)
  private
     { Private declarations }
  protected
     procedure Execute; override;
  public
     Jpeg_Stream: TMemoryStream; //   jpeg
     Active:integer;             //   (=1   )
     //  
     result:boolean;             //    
     result_Bitmap:TBitmap;
     tim_Assign:integer;
  end;

implementation

procedure IJLCheck(Code:Integer);
begin
 if Code<>IJL_OK then
   raise EIJLException.Create(ijlErrorStr(Code));
end;

const
   ScaleMul:array[TIJLScale] of DWORD=(1,2,4,8);

procedure LoadBmpFromJpegStream(var bitmap: TBitmap;  Stream:TMemoryStream;
 const SetPixelFormat:boolean=true; const Scale:TIJLScale=ijlFull);
const
 ScaleBuffIOType:array[TIJLScale] of TIJLIOTYPE=(IJL_JBUFF_READWHOLEIMAGE,IJL_JBUFF_READONEHALF,IJL_JBUFF_READONEQUARTER,IJL_JBUFF_READONEEIGHTH);
var
 jcprops: TJPEG_CORE_PROPERTIES;
 DIB: TDIBSection;
begin
 if not Assigned (Bitmap) then Exit;

 FillChar(jcprops,SizeOf(jcprops),0);
 IJLCheck(ijlInit(@jcprops));
 try
   jcprops.JPGBytes := PByte(Stream.Memory);
   jcprops.JPGSizeBytes:= Stream.Size;
   IJLCheck(ijlRead(@jcprops, IJL_JBUFF_READPARAMS));

   if (jcprops.JPGChannels<>1) and (jcprops.JPGChannels<>3) then
     raise EIJLException.CreateFmt('Cannot load JPEG with %d Channels',[jcprops.JPGChannels]);

//   Bitmap.Width:=0;
//   Bitmap.Height:=0;

   if SetPixelFormat then
   begin
     jcprops.DIBChannels:=jcprops.JPGChannels;
     if jcprops.DIBChannels=1 then
       Bitmap.PixelFormat:= pf8Bit
     else
       Bitmap.PixelFormat:= pf24Bit;
   end else
   begin
     case Bitmap.PixelFormat of
       pf8Bit:  jcprops.DIBChannels:= 1;
       pf24Bit: jcprops.DIBChannels:= 3;
       else raise EIJLException.Create('Cannot load in bitmap with specified PixelFormat');
     end;
   end;
   if jcprops.DIBChannels=1 then
     jcprops.DIBColor:= IJL_G
   else
     jcprops.DIBColor:= IJL_BGR;

   Bitmap.Width :=(jcprops.JPGWidth + ScaleMul[Scale]-1) div ScaleMul[Scale];
   Bitmap.Height:=(jcprops.JPGHeight+ ScaleMul[Scale]-1) div ScaleMul[Scale];

   FillChar(DIB, SizeOf (DIB), 0);
   Win32Check(GetObject(Bitmap.Handle, SizeOf(DIB),@DIB)<>0);
   jcprops.DIBBytes := PByte(DIB.dsBm.bmBits);
   jcprops.DIBWidth := DIB.dsBm.bmWidth;
   jcprops.DIBHeight:= -DIB.dsBm.bmHeight;
   jcprops.DIBPadBytes:=((jcprops.DIBWidth*jcprops.DIBChannels+3) and -$04)-(jcprops.DIBWidth*jcprops.DIBChannels);

   IJLCheck(ijlRead(@jcprops, ScaleBuffIOType[Scale]));
 finally
   IJLCheck(ijlFree(@jcprops));
 end;
 Bitmap.Modified:=True;
end;      


procedure TBMP_To_Jpeg_Thread.Execute;
var tim_vr:int64;
begin  
    repeat
         if Active=1 then
           begin
              result:=false;
              if Jpeg_Stream<>nil then
                 begin
                    try
                       tim_vr:=GetTickCount;
                       try
                             if result_Bitmap=nil then
                                  result_Bitmap:=TBitmap.Create;
                             Jpeg_Stream.Position:=0;
                             LoadBmpFromJpegStream(result_Bitmap,Jpeg_Stream,true);

                       finally
                             Jpeg_Stream.Free;
                             Jpeg_Stream:=nil;
                       end;
                       tim_Assign:=GetTickCount-tim_vr;
                       result:=true;
                    except
                       result:=false;
                       try
                           Jpeg_Stream.Free;
                       except
                           Jpeg_Stream:=nil;
                       end;
                    end;
                 end;
              Active:=2;
              sleep(2);
           end;
        sleep(1);
    until Terminated;
    if Jpeg_Stream<>nil then
       begin
           try
              Jpeg_Stream.Free;
              Jpeg_Stream:=nil;
           except
           end;
       end;
    if result_Bitmap<>nil then
       begin
           try
              result_Bitmap.Free;
              result_Bitmap:=nil;
           except
           end;
       end;
end;

end.
