unit jpeg_to_bmp_unit;

interface

uses Windows, SysUtils, Classes, Graphics, IJL;

type
 EIJLException = class(Exception);

type
 TIJLScale=(ijlFull, ijlHalf, ijlQuarter, ijlEighth);


   procedure LoadBmpFromJpegStream(var bitmap: TBitmap;  Stream:TMemoryStream; const SetPixelFormat:boolean=true; const Scale:TIJLScale=ijlFull);

implementation

procedure IJLCheck(Code:Integer);
begin
 if Code<>IJL_OK then
   raise EIJLException.Create(ijlErrorStr(Code));
end;

const
   ScaleMul:array[TIJLScale] of DWORD=(1,2,4,8);

procedure LoadBmpFromJpegStream(var bitmap: TBitmap;  Stream:TMemoryStream;
 const SetPixelFormat:boolean=true; const Scale:TIJLScale=ijlFull);
const
 ScaleBuffIOType:array[TIJLScale] of TIJLIOTYPE=(IJL_JBUFF_READWHOLEIMAGE,IJL_JBUFF_READONEHALF,IJL_JBUFF_READONEQUARTER,IJL_JBUFF_READONEEIGHTH);
var
 jcprops: TJPEG_CORE_PROPERTIES;
 DIB: TDIBSection;
begin
 if not Assigned (Bitmap) then Exit;

 FillChar(jcprops,SizeOf(jcprops),0);
 IJLCheck(ijlInit(@jcprops));
 try
   jcprops.JPGBytes := PByte(Stream.Memory);
   jcprops.JPGSizeBytes:= Stream.Size;
   IJLCheck(ijlRead(@jcprops, IJL_JBUFF_READPARAMS));

   if (jcprops.JPGChannels<>1) and (jcprops.JPGChannels<>3) then
     raise EIJLException.CreateFmt('Cannot load JPEG with %d Channels',[jcprops.JPGChannels]);

//   Bitmap.Width:=0;
//   Bitmap.Height:=0;

   if SetPixelFormat then
   begin
     jcprops.DIBChannels:=jcprops.JPGChannels;
     if jcprops.DIBChannels=1 then
       Bitmap.PixelFormat:= pf8Bit
     else
       Bitmap.PixelFormat:= pf24Bit;
   end else
   begin
     case Bitmap.PixelFormat of
       pf8Bit:  jcprops.DIBChannels:= 1;
       pf24Bit: jcprops.DIBChannels:= 3;
       else raise EIJLException.Create('Cannot load in bitmap with specified PixelFormat');
     end;
   end;
   if jcprops.DIBChannels=1 then
     jcprops.DIBColor:= IJL_G
   else
     jcprops.DIBColor:= IJL_BGR;

   Bitmap.Width :=(jcprops.JPGWidth + ScaleMul[Scale]-1) div ScaleMul[Scale];
   Bitmap.Height:=(jcprops.JPGHeight+ ScaleMul[Scale]-1) div ScaleMul[Scale];

   FillChar(DIB, SizeOf (DIB), 0);
   Win32Check(GetObject(Bitmap.Handle, SizeOf(DIB),@DIB)<>0);
   jcprops.DIBBytes := PByte(DIB.dsBm.bmBits);
   jcprops.DIBWidth := DIB.dsBm.bmWidth;
   jcprops.DIBHeight:= -DIB.dsBm.bmHeight;
   jcprops.DIBPadBytes:=((jcprops.DIBWidth*jcprops.DIBChannels+3) and -$04)-(jcprops.DIBWidth*jcprops.DIBChannels);

   IJLCheck(ijlRead(@jcprops, ScaleBuffIOType[Scale]));
 finally
   IJLCheck(ijlFree(@jcprops));
 end;
 Bitmap.Modified:=True;
end;

end.
